==> apb_2.12-1.0.6-sources.jar: org/mulesoft/apb/client/scala/APBClient.scala <==
package org.mulesoft.apb.client.scala

import amf.apicontract.client.scala.{AMFConfiguration, UnrecognizedSpecException, WebAPIConfiguration}
import amf.core.client.common.transform.PipelineId
import amf.core.client.scala.config.RenderOptions
import amf.core.client.scala.model.document.{BaseUnit, ExternalFragment}
import amf.core.client.scala.validation.{AMFValidationReport, AMFValidationResult}
import amf.core.client.scala.{AMFParseResult, AMFResult}
import amf.core.internal.remote.Spec
import amf.core.internal.unsafe.PlatformSecrets
import amf.core.internal.validation.CoreParserValidations.CouldntGuessRoot
import amf.custom.validation.client.ProfileValidatorNodeBuilder
import amf.custom.validation.internal.report.parser.EmptyProfile
import org.mulesoft.apb.project.client.scala.ProjectConfiguration
import org.mulesoft.apb.project.client.scala.environment.DependencyFetcher
import org.mulesoft.apb.project.client.scala.model.{Gav, ProjectDescriptor}

import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent.Future
import scala.language.postfixOps

class APBClient private[apb] (val projectConfiguration: ProjectConfiguration) {

  val descriptor: ProjectDescriptor = projectConfiguration.descriptor

  implicit class AMFHelper(config: AMFConfiguration) {
    def parse(): Future[AMFParseResult]  = config.baseUnitClient().parse(descriptor.main)
    def resolve(bu: BaseUnit): AMFResult = config.baseUnitClient().transform(bu, PipelineId.Editing)

    def validate(bu: BaseUnit): Future[AMFValidationReport] = config.baseUnitClient().validate(bu)
  }

  implicit class AMFResultHelper(result: AMFResult) {
    def addResults(other: AMFResult): AMFResult = result.copy(results = result.results ++ other.results)
  }

  private def parse(): Future[(AMFParseResult, Spec)] = {
    projectConfiguration.projectParseConfig.parse().map(u => (u, u.sourceSpec))
  }

  private def resolve(r: AMFResult, spec: Spec): AMFResult = projectConfiguration.configFor(spec).resolve(r.baseUnit)

  private def validate(r: AMFResult, spec: Spec): Future[AMFValidationReport] =
    projectConfiguration.configFor(spec).validate(r.baseUnit.cloneUnit())

  private def serialize(bu: BaseUnit, withSourceMaps: Boolean = false): String = {
    val options = if (withSourceMaps) RenderOptions().withSourceMaps else RenderOptions()
    projectConfiguration
      .dependenciesConfig(WebAPIConfiguration.WebAPI().withRenderOptions(options))
      .baseUnitClient()
      .render(bu)
  }

  private def lint(unit: BaseUnit): Future[AMFValidationReport] =
    for {
      reports <- {
        val jsonld = serialize(unit, withSourceMaps = true)
        val eventualReports: Seq[Future[AMFValidationReport]] = projectConfiguration.profileDependencies.toList
          .map { profileDep =>
            ProfileValidatorNodeBuilder
              .validator(profileDep.profile)
              .validate(jsonld, descriptor.main)
          }
        Future.sequence(eventualReports)
      }
    } yield {
      val empty = AMFValidationReport.empty(unit.id, EmptyProfile)
      reports.foldLeft(empty)((acc, curr) => acc.merge(curr))
    }

  def getAMFConfiguration: AMFConfiguration = projectConfiguration.projectParseConfig

  def build(): Future[AMFResult] = {
    for {
      (result, spec) <- parse()
    } yield {
      try {
        val transformConfig = projectConfiguration.configFor(spec)
        val transformResult = transformConfig.resolve(result.baseUnit)
        transformResult.copy(results = transformResult.results ++ result.results)
      } catch {
        case _: UnrecognizedSpecException =>

          // Should we reaffirm the parsing result with these assertions?
          //
          // Note: `configFor(spec) will throw UnrecognizedSpecException for specs other than RAML08, RAML10, OAS20, OAS30
          // this includes Async, GraphQL, GRPC

//          assert(result.baseUnit.isInstanceOf[ExternalFragment], "")
//          assert(result.results.length == 1, "")
//          assert(result.results.head.validationId == CouldntGuessRoot.id, "")
          result
      }
    }
  }

  def compile(): Future[AMFParseResult] = parse().map(_._1)

  def report(): Future[AMFValidationReport] = {
    for {
      (parseResult, spec) <- parse()
      resolveResult       <- Future.successful(resolve(parseResult, spec))
      validationReport <- {
        validate(resolveResult, spec)
      }
      lintReport <- lint(resolveResult.baseUnit)
    } yield validationReport.copy(
        results = validationReport.results ++ parseResult.results ++ resolveResult.results ++ lintReport.results
    )
  }

  def lint(): Future[AMFValidationReport] = build().flatMap(b => lint(b.baseUnit))

  def serialize(): Future[String] = build().map(r => serialize(r.baseUnit))

  def checkTree(): Seq[AMFValidationResult] = projectConfiguration.errors.allErrors

  lazy val listDependencies: Seq[Gav] = projectConfiguration.allDependencies.map(_.descriptor.gav)

}

object APBClient extends PlatformSecrets {

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromContent(content: String, dependencyFetcher: DependencyFetcher, base: Option[String]): Future[APBClient] = {
    APBClientBuilder(dependencyFetcher).buildFromContent(content, base)
  }

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromDirectory(directory: String, dependencyFetcher: DependencyFetcher): Future[APBClient] = {
    APBClientBuilder(dependencyFetcher).build(directory)
  }

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromProjectConfiguration(projectConfiguration: ProjectConfiguration): APBClient = {
    new APBClient(projectConfiguration)
  }

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromDescriptor(descriptor: ProjectDescriptor, dependencyFetcher: DependencyFetcher): Future[APBClient] = {
    APBClientBuilder(dependencyFetcher).build(descriptor)
  }
}

==> apb_2.12-1.6.0-sources.jar: org/mulesoft/apb/client/scala/APBClient.scala <==
package org.mulesoft.apb.client.scala

import amf.apicontract.client.scala.{AMFConfiguration, APIConfiguration, UnrecognizedSpecException}
import amf.core.client.common.transform.PipelineId
import amf.core.client.scala.config.RenderOptions
import amf.core.client.scala.model.document.BaseUnit
import amf.core.client.scala.validation.{AMFValidationReport, AMFValidationResult}
import amf.core.client.scala.{AMFParseResult, AMFResult}
import amf.core.internal.remote.Spec
import amf.core.internal.unsafe.PlatformSecrets
import amf.custom.validation.client.ProfileValidatorNodeBuilder
import amf.custom.validation.internal.report.parser.EmptyProfile
import org.mulesoft.apb.project.client.scala.ProjectConfiguration
import org.mulesoft.apb.project.client.scala.environment.DependencyFetcher
import org.mulesoft.apb.project.client.scala.model.{Gav, ProjectDescriptor}

import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent.Future
import scala.language.postfixOps

class APBClient private[apb] (val projectConfiguration: ProjectConfiguration) {

  val descriptor: ProjectDescriptor = projectConfiguration.descriptor

  implicit class AMFHelper(config: AMFConfiguration) {
    def parse(): Future[AMFParseResult]  = config.baseUnitClient().parse(descriptor.main)
    def resolve(bu: BaseUnit): AMFResult = config.baseUnitClient().transform(bu, PipelineId.Editing)

    def validate(bu: BaseUnit): Future[AMFValidationReport] = config.baseUnitClient().validate(bu)
  }

  implicit class AMFResultHelper(result: AMFResult) {
    def addResults(other: AMFResult): AMFResult = result.copy(results = result.results ++ other.results)
  }

  private def parse(): Future[(AMFParseResult, Spec)] = {
    projectConfiguration.projectParseConfig.parse().map(u => (u, u.sourceSpec))
  }

  private def resolve(r: AMFResult, spec: Spec): AMFResult = projectConfiguration.configFor(spec).resolve(r.baseUnit)

  private def validate(r: AMFResult, spec: Spec): Future[AMFValidationReport] =
    projectConfiguration.configFor(spec).validate(r.baseUnit.cloneUnit())

  private def serialize(bu: BaseUnit, withSourceMaps: Boolean = false, withSourceInformation: Boolean = false): String = {
    var options = RenderOptions()
    if (withSourceMaps) options = options.withSourceMaps
    if (withSourceInformation) options = options.withSourceInformation
    serialize(bu, options)
  }

  private def serialize(bu: BaseUnit, renderOptions: RenderOptions): String = {
    projectConfiguration
      .dependenciesConfig(APIConfiguration.API().withRenderOptions(renderOptions))
      .baseUnitClient()
      .render(bu)
  }

  private def lint(unit: BaseUnit): Future[AMFValidationReport] =
    for {
      reports <- {
        val jsonld = serialize(unit, withSourceMaps = true, withSourceInformation = true)
        val eventualReports: Seq[Future[AMFValidationReport]] = projectConfiguration.profileDependencies.toList
          .map { profileDep =>
            ProfileValidatorNodeBuilder
              .validator(profileDep.profile)
              .validate(jsonld, descriptor.main)
          }
        Future.sequence(eventualReports)
      }
    } yield {
      val empty = AMFValidationReport.empty(unit.id, EmptyProfile)
      reports.foldLeft(empty)((acc, curr) => acc.merge(curr))
    }

  def getAMFConfiguration: AMFConfiguration = projectConfiguration.projectParseConfig

  def build(): Future[AMFResult] = {
    for {
      (result, spec) <- parse()
    } yield {
      try {
        val transformConfig = projectConfiguration.configFor(spec)
        val transformResult = transformConfig.resolve(result.baseUnit)
        transformResult.copy(results = transformResult.results ++ result.results)
      } catch {
        case _: UnrecognizedSpecException =>
          // Should we reaffirm the parsing result with these assertions?
          //
          // Note: `configFor(spec) will throw UnrecognizedSpecException for specs other than RAML08, RAML10, OAS20, OAS30
          // this includes Async, GraphQL, GRPC

//          assert(result.baseUnit.isInstanceOf[ExternalFragment], "")
//          assert(result.results.length == 1, "")
//          assert(result.results.head.validationId == CouldntGuessRoot.id, "")
          result
      }
    }
  }

  def compile(): Future[AMFParseResult] = parse().map(_._1)

  def report(): Future[AMFValidationReport] = {
    for {
      (parseResult, spec) <- parse()
      resolveResult       <- Future.successful(resolve(parseResult, spec))
      validationReport <- {
        validate(resolveResult, spec)
      }
      lintReport <- lint(resolveResult.baseUnit)
    } yield validationReport.copy(
        results = validationReport.results ++ parseResult.results ++ resolveResult.results ++ lintReport.results
    )
  }

  def lint(): Future[AMFValidationReport] = build().flatMap(b => lint(b.baseUnit))

  def serialize(): Future[String] = build().map(r => serialize(r.baseUnit))

  def serialize(renderOptions: RenderOptions): Future[String] = build().map(r => serialize(r.baseUnit, renderOptions))

  def checkTree(): Seq[AMFValidationResult] = projectConfiguration.errors.allErrors

  lazy val listDependencies: Seq[Gav] = projectConfiguration.allDependencies.map(_.descriptor.gav)

}

object APBClient extends PlatformSecrets {

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromContent(content: String, dependencyFetcher: DependencyFetcher, base: Option[String]): Future[APBClient] = {
    APBClientBuilder(dependencyFetcher).buildFromContent(content, base)
  }

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromDirectory(directory: String, dependencyFetcher: DependencyFetcher): Future[APBClient] = {
    APBClientBuilder(dependencyFetcher).build(directory)
  }

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromProjectConfiguration(projectConfiguration: ProjectConfiguration): APBClient = {
    new APBClient(projectConfiguration)
  }

  @deprecated("Prefer using org.mulesoft.apb.client.platform.APBClientBuilder instead", "1.0.5")
  def fromDescriptor(descriptor: ProjectDescriptor, dependencyFetcher: DependencyFetcher): Future[APBClient] = {
    APBClientBuilder(dependencyFetcher).build(descriptor)
  }
}
